/*

  Copyright (c) 2016 Steffen Illhardt

  Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the
  "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish,
  distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to
  the following conditions:

  The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
  MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
  CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
  SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

*/
/** @file mousepos.c
 * Determines the current mouse position and returns the coordinates of the character cell where the mouse pointer is placed over.
 * The returned value has a width of 32 bits where the 16 low bits represent the x position and the 16 high bits represent the y position
 * The program returns 0 (x=0 and y=0) if the mouse pointer is outside of the visible area of the console window buffer.
 * In order to be able to distiguish between positions inside and outside of the window, the buffer origin (top left character cell of the
 *  window buffer) was defined to have coordinates x=1 and y=1.
 */
#ifdef _WIN32_WINNT
#undef _WIN32_WINNT
#endif
#define _WIN32_WINNT 0x0600
#include <windows.h>

int main(void)
{
  HWND hWnd = GetConsoleWindow();
  RECT rct = {0};
  POINT pt = {0};
  SCROLLBARINFO unused = {sizeof(SCROLLBARINFO)};
  GetWindowRect(hWnd, &rct);
  GetCursorPos(&pt);
  if (pt.x < rct.left + GetSystemMetrics(SM_CXSIZEFRAME)
      || pt.x > rct.right - GetSystemMetrics(SM_CXSIZEFRAME) - (GetScrollBarInfo(hWnd, OBJID_VSCROLL, &unused) ? GetSystemMetrics(SM_CXVSCROLL) : 0)
      || pt.y < rct.top + GetSystemMetrics(SM_CYCAPTION) + GetSystemMetrics(SM_CYSIZEFRAME)
      || pt.y > rct.bottom - GetSystemMetrics(SM_CYSIZEFRAME) - (GetScrollBarInfo(hWnd, OBJID_HSCROLL, &unused) ? GetSystemMetrics(SM_CYHSCROLL) : 0))
    return 0;

  HANDLE hOut = GetStdHandle(STD_OUTPUT_HANDLE);
  CONSOLE_FONT_INFO cfi = {0};
  GetCurrentConsoleFont(hOut, FALSE, &cfi);
  COORD cfs = GetConsoleFontSize(hOut, cfi.nFont);
  ScreenToClient(hWnd, &pt);
  return (DWORD)(WORD)(pt.y / cfs.Y + GetScrollPos(hWnd, SB_VERT) + 1) << 16u | (WORD)(pt.x / cfs.X + GetScrollPos(hWnd, SB_HORZ) + 1);
}
